import sys

import click

FOREIGN_KEYS = [
    ("items", "parent_id", "items", "id"),
    ("items", "project_id", "projects", "id"),
    ("notes", "item_id", "items", "id"),
    ("notes", "project_id", "projects", "id"),
    ("projects", "parent_id", "projects", "id"),
    ("users", "inbox_project", "projects", "id"),
]


def error(message):
    click.secho(message, bold=True, fg="red")
    sys.exit(-1)


def add_foreign_keys(db, tables=None):
    for fk in FOREIGN_KEYS:
        if tables and fk[0] not in tables:
            continue
        db.add_foreign_keys([fk])



import time
import click
import pathlib
import json
import sqlite_utils
from tqdm import tqdm
from todoist_to_sqlite import utils
from pytodoist import todoist
from pytodoist.api import TodoistAPI


@click.group()
@click.version_option()
def cli():
    "Save data from Todoist to a SQLite database"


@cli.command()
@click.option(
    "-a",
    "--auth",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
    default="auth.json",
    help="Path to save tokens to, defaults to ./auth.json.",
)
def auth(auth):
    "Save authentication credentials to a JSON file"
    auth_data = {}
    if pathlib.Path(auth).exists():
        auth_data = json.load(open(auth))
    click.echo(
        "In Todoist, navigate to Settings > Integrations > API Token and paste it here:"
    )
    personal_token = click.prompt("API Token")
    auth_data["todoist_api_token"] = personal_token
    open(auth, "w").write(json.dumps(auth_data, indent=4) + "\n")
    click.echo()
    click.echo(
        "Your authentication credentials have been saved to {}. You can now import tasks by running".format(
            auth
        )
    )
    click.echo()
    click.echo("    todoist-to-sqlite tasks todoist.db")
    click.echo()


@cli.command()
@click.argument(
    "db_path",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
    required=True,
)
@click.option(
    "-a",
    "--auth",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
    default="auth.json",
    help="Path to save tokens to, defaults to auth.json",
)
def sync(db_path, auth):
    """Sync todoist data for the authenticated user"""
    db = sqlite_utils.Database(db_path)
    try:
        data = json.load(open(auth))
        token = data["todoist_api_token"]
    except (KeyError, FileNotFoundError):
        utils.error(
            "Cannot find authentication data, please run `todoist_to_sqlite auth`!"
        )
    api = TodoistAPI()
    sync_data = api.sync(api_token=token, sync_token="*").json()
    for category in ["items", "labels", "projects", "filters", "notes", "sections"]:
        db[category].upsert_all(sync_data[category], pk="id", alter=True)

    db["users"].upsert_all(sync_data["collaborators"], pk="id")
    db["users"].upsert(sync_data["user"], pk="id")

    utils.add_foreign_keys(db)


@cli.command()
@click.argument(
    "db_path",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
    required=True,
)
@click.option(
    "-a",
    "--auth",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
    default="auth.json",
    help="Path to save tokens to, defaults to auth.json",
)
def completed_tasks(db_path, auth):
    """Save all completed tasks for the authenticated user (requires Todoist premium)"""
    db = sqlite_utils.Database(db_path)
    try:
        data = json.load(open(auth))
        token = data["todoist_api_token"]
    except (KeyError, FileNotFoundError):
        utils.error(
            "Cannot find authentication data, please run `todoist_to_sqlite auth`!"
        )
    api = TodoistAPI()
    stats = api.get_productivity_stats(token)
    PAGE_SIZE = 50
    offset = 0

    progress_bar = tqdm(
        desc="Fetching completed tasks", total=stats.json().get("completed_count")
    )

    while True:
        data = api.get_all_completed_tasks(
            api_token=token, limit=PAGE_SIZE, offset=offset
        ).json()

        db["items"].upsert_all(data["items"], pk="id", alter=True)
        db["projects"].upsert_all(
            data["projects"].values(), pk="id", alter=True)

        num_items = len(data["items"])
        if num_items == 0 or True:
            break

        progress_bar.update(num_items)
        offset += num_items
        time.sleep(1)

    progress_bar.close()
    utils.add_foreign_keys(db, tables=["users", "projects"])


if __name__ == "__main__":
    cli()



import os

from setuptools import setup

VERSION = "0.1"


def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()


setup(
    name="todoist-to-sqlite",
    description="Save data from Todoist to a SQLite database",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Benjamin Congdon",
    author_email="me@bcon.gdn",
    url="https://github.com/bcongdon/todoist-to-sqlite",
    project_urls={
        "Source": "https://github.com/bcongdon/todoist-to-sqlite",
        "Issues": "https://github.com/bcongdon/todoist-to-sqlite/issues",
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Database",
    ],
    keywords="todoist sqlite export dogsheep",
    version=VERSION,
    packages=["todoist_to_sqlite"],
    entry_points="""
        [console_scripts]
        todoist-to-sqlite=todoist_to_sqlite.cli:cli
    """,
    install_requires=[
        "click",
        "requests",
        "sqlite-utils~=1.11",
        "tqdm~=4.36",
        "pytodoist~=2.1",
    ],
    extras_require={"test": ["pytest"]},
    tests_require=["todoist-to-sqlite[test]"],
)
